@@ -36,34 +36,64 @@ class Charm(object):
                 os.path.exists(os.path.join(self.branch, '.git')):
             return Git(self.path, self.branch, self.log)
         elif self.branch.startswith("bzr") or self.branch.startswith('lp:') \
-                or os.path.exists(os.path.join(self.branch, '.bzr')):
+                or os.path.exists(os.path.join(self.branch, '.bzr')) \
+                or self.branch.startswith('file:'):
             return Bzr(self.path, self.branch, self.log)
 
     @classmethod
-    def from_service(cls, name, series_path, d):
-        branch, rev = None, None
-        charm_branch = d.get('branch')
+    def from_service(cls, name, repo_path, deploy_series, data):
+        """
+        name: service name
+        data['charm']: charm name or store charm url
+        data['charm_url'] store charm url
+        """
+        branch, rev, series = None, None, None
+        charm_branch = data.get('branch')
         if charm_branch is not None:
             branch, sep, rev = charm_branch.partition('@')
 
         charm_path, store_url, build = None, None, None
-        name = d.get('charm', name)
+        name = data.get('charm', name)
         if name.startswith('cs:'):
             store_url = name
+        elif name.startswith('local:'):
+            # Support vcs charms specifying their
+            parts = name[len('local:'):].split('/')
+            if len(parts) == 2:
+                series, name = parts
+            elif data.get('series'):
+                series = data['series']
+                name = parts.pop()
+            else:
+                series = deploy_series
+            charm_path = path_join(repo_path, series, name)
+        elif 'series' in data:
+            series = data['series']
+            charm_path = path_join(repo_path, series, name)
         else:
-            charm_path = path_join(series_path, name)
-            build = d.get('build', '')
+            charm_path = path_join(repo_path, deploy_series, name)
+
         if not store_url:
-            store_url = d.get('charm_url', None)
+            store_url = data.get('charm_url', None)
 
         if store_url and branch:
             cls.log.error(
                 "Service: %s has both charm url: %s and branch: %s specified",
                 name, store_url, branch)
+        if not store_url and not branch:
+            cls.log.error(
+                "Service: %s has neither charm url or branch specified",
+                name)
+        elif not store_url:
+            build = data.get('build', '')
+
         return cls(name, charm_path, branch, rev, build, store_url)
 
     def is_local(self):
-        return not self._charm_url
+        if self._charm_url:
+            if self._charm_url.startswith('cs:'):
+                return False
+        return True
 
     def exists(self):
         return self.is_local() and path_exists(self.path)
@@ -96,6 +126,8 @@ class Charm(object):
             return
         self.log.debug(" Branching charm %s @ %s", self.branch, self.path)
         self.vcs.branch()
+        if self.rev:
+            self.vcs.update(self.rev)
         self.build()
 
     @property
@@ -104,6 +136,12 @@ class Charm(object):
             self._path = self._get_charm_store_cache()
         return self._path
 
+    @property
+    def series_path(self):
+        if not self.is_local():
+            return None
+        return os.path.dirname(self.path)
+
     def _fetch_store_charm(self, update=False):
         cache_dir = self._get_charm_store_cache()
         self.log.debug("Cache dir %s", cache_dir)
