@@ -37,8 +37,10 @@ class GoEnvironment(BaseEnvironment):
                 token = data.get('environments', {}).get(self.name, {}).get(
                     'admin-secret')
                 if token is None:
-                    raise EnvError({
-                        'Error': "Environment config not found %s" % self.name})
+                    raise EnvError(
+                        {'Error': "Environment config not found %s" % self.name
+                         })
+                return token
         with open(jenv) as fh:
             data = yaml_load(fh.read())
             token = data.get('bootstrap-config', {}).get(
@@ -69,8 +71,10 @@ class GoEnvironment(BaseEnvironment):
             try:
                 self.client = EnvironmentClient(self.api_endpoint)
             except socket.error as err:
-                if err.errno != errno.ETIMEDOUT:
+                if not err.errno in (
+                        errno.ETIMEDOUT, errno.ECONNREFUSED, errno.ECONNRESET):
                     raise
+                time.sleep(1)
                 continue
             else:
                 break
